/*defrad.h*/

#ifndef __defrad_INCLUDED
#define __defrad_INCLUDED


/**															   
Instruction	Instruction Format [binary]	#Data Bytes		Operation					   
---------------------------------------------------------------------------------------------------------------------------
R_REGISTER	000A AAAA 			1 to 5 LSByte first 	Read registers. AAAAA = 5 bit Memory Map Address   
W_REGISTER	001A AAAA 			1 to 5 LSByte first	Write registers. AAAAA = 5 bit Memory Map Address. 
									Executable in power down or standby modes only.	   
R_RX_PAYLOAD 	0110 0001 			1 to 32 LSByte first	Read RX-payload: 1  32 bytes.			   
									A read operation will always start at byte 0.	   
									Payload will be deleted from FIFO after it is read.
									Used in RX mode.				   
W_TX_PAYLOAD 	1010 0000 			1 to 32 LSByte first	Used in TX mode.				   
									Write TX-payload: 1  32 bytes.			   
									A write operation will always start at byte 0.	   
FLUSH_TX	1110 0001 			0			Flush TX FIFO, used in TX mode			   
FLUSH_RX	1110 0010 			0			Flush RX FIFO, used in RX mode.			   
									Should not be executed during transmission of	   
									acknowledge, i.e. acknowledge package will not	   
									be completed.					   
REUSE_TX_PL 	1110 0011 			0			Used for a PTX device.				   
									Reuse last sent payload.			   
									Packets will be repeatedly resent as ong as CE 	   
									is high. TX payload reuse is active until 	   
									W_TX_PAYLOAD or FLUSH TX is executed.		   
									TX payload reuse must not be activated or 	   
									deactivated during package transmission.	   
NOP		1111 1111 			0			No Operation.					   
									May be used to read the STATUS register 	   
--------------------------------------------------------------------------------------------------------------------------*/
/** SPI instructions */
#define R_REGISTER(addr) 	(addr & 0x1F)
#define W_REGISTER(addr) 	((addr|0x20) & 0x3F)
#define R_RX_PAYLOAD 		0x61
#define W_TX_PAYLOAD 		0xA0
#define FLUSH_TX 		0xE1
#define FLUSH_RX 		0xE2
#define REUSE_TX_PL 		0xE3
#define NOP 			0xFF



/* Define nRF24L01 interrupt flag's*/
#define IDLE		0x00
#define MAX_RT		0x10
#define TX_DS		0x20
#define RX_DR		0x40
#define TX_FULL		0x01


/********************************************************************************************************************/
/* SPI(nRF24L01) registers(addresses)*/
#define CONFIG		0x00
#define EN_AA		0x01
#define EN_RXADDR	0x02
#define SETUP_AW	0x03
#define SETUP_RETR	0x04
#define RF_CH		0x05
#define RF_SETUP	0x06
#define STATUS		0x07
#define OBSERVE_TX	0x08
#define CD		0x09
#define RX_ADDR_P0	0x0A
#define RX_ADDR_P1	0x0B
#define RX_ADDR_P2	0x0C
#define RX_ADDR_P3	0x0D
#define RX_ADDR_P4	0x0E
#define RX_ADDR_P5	0x0F
#define TX_ADDR		0x10
#define RX_PW_P0	0x11
#define RX_PW_P1	0x12
#define RX_PW_P2	0x13
#define RX_PW_P3	0x14
#define RX_PW_P4	0x15
#define RX_PW_P5	0x16
#define FIFO_STATUS	0x17


#define TX_ADR_LENGTH	5
#define TX_PLOAD_WIDTH	32

#endif