/*************************************************************************
*                            MUlist.c
*                            ------
*  Detta objekt inehller listan med MU's och dess listor med samples.
*  Den kan ven stta och getta mu's samt samples.
****************************************************************************/
/**
* Includes
*/
#include "INC/stdio.h"

#include "INC/inttypes.h"
#include "INC/defines.h"
#include "INC/defcom.h"
#include "INC/defmulist.h"
#include "INC/r_MUlist.h"



/**
* Private function declarations
*/
uint8_t muAddInitSampl(void);
uint8_t sampAlloc(void);
void sampFree(uint8_t remSamp);

/**
* Private variables
*/
mu muList[8];
uint8_t addrcnt=0;
sampList sampArray[200];
uint8_t sampFirstFree = 0;

/***********************************************************************************
* Private functions
************************************************************************************/
uint8_t sampAlloc(void)
{
	uint8_t retval = sampFirstFree;

	/*Sno frsta lediga sample och peka runt mm.*/
	sampArray[sampArray[sampFirstFree].next].prev = sampArray[sampFirstFree].prev;
	sampArray[sampArray[sampFirstFree].prev].next = sampArray[sampFirstFree].next;
	sampFirstFree = sampArray[sampFirstFree].next;
	
	/*Om vi har slut p minne, s sno lite frn genomsnittsvrdena av temperaturen.
	  Vilken som ska snos borde bli dynamiskt s att den ldsta alltid plockas, men
	  det fr bli nr tid finns...*/
	if(sampArray[sampFirstFree].next == sampFirstFree){
		sampFree(muList[0].witSampl[0]);
		muList[0].witSampl[0] = sampArray[muList[0].witSampl[0]].next;
	}
	
	return retval;
}

void sampFree(uint8_t remSamp)
{
	/*Lnka runt i ursprungslistan*/
	sampArray[sampArray[remSamp].prev].next = sampArray[remSamp].next;
	sampArray[sampArray[remSamp].next].prev = sampArray[remSamp].prev;

	/*Lnka in i sampFirstFree listan*/
	sampArray[remSamp].prev = sampArray[sampFirstFree].prev;
	sampArray[remSamp].next = sampFirstFree;
	sampArray[sampFirstFree].prev = remSamp;
	sampArray[sampArray[sampFirstFree].prev].next = remSamp;
}

/***********************************************************************************
* Public functions
************************************************************************************/
/**
* Initiation of MU lists and sample lists
*/
void muInit()
{
	int i=0;
	int j=0;
	for(i=0;i<8;i++){
		muList[i].witAvail = 0;
		muList[i].witEnabl = 0;
		/*0x10 is used to indicate uninitiated mu. Since address is 4bit
		  long, 0x10 is outside address range and therefore not valid as
		  an address. */
		/*When connected, address is set 0-15*/
		muList[i].address = 0x10;
		
		for(i=0;i<200;i++){
			sampArray[i].next = i+1;
			sampArray[i+1].prev = i;
		}
		sampArray[0].prev = 199;
		sampArray[199].next = 0;
		
		for(i=0;i<8;i++){
			for(j=0;j<=10;j++){
				/*set initial sample to all WIT's*/
				muList[i].witSampl[j] = muAddInitSampl();
			}
		}
	}
	muUpdateUnitList();
}

void muClearSamples(uint8_t mu)
{
	int j=0;
	
	for(j=0;j<=10;j++){
		while(muList[mu].witSampl[j] != sampArray[muList[mu].witSampl[j]].next){
			sampFree(muList[mu].witSampl[j]);
		}
		sampArray[muList[mu].witSampl[j]].value = 0;
		sampArray[muList[mu].witSampl[j]].timeY = tmrGetYear();
		sampArray[muList[mu].witSampl[j]].timeMo = tmrGetMonth();
		sampArray[muList[mu].witSampl[j]].timeD = tmrGetDay();
		sampArray[muList[mu].witSampl[j]].timeH = tmrGetHour();
		sampArray[muList[mu].witSampl[j]].timeM = tmrGetMinute();
		sampArray[muList[mu].witSampl[j]].timeS = tmrGetSecond();
	}
}

/**
* Ask every unit availible (mu[i].address != 0x10) in the list for status and make a broadcast for
* unknown units that can be added to the list. When adding a unit, give it the address addrcnt
* and increment it. The added unit shall be softly turned of!
*/
void muUpdateUnitList(void)
{
	/*0x10 is used to indicate uninitiated mu. Since address is 4bit
	long, 0x10 is outside address range and therefore not valid as an address.
	When connected, address is set 0-15*/
	
	/*///TODO: Send a FIND_MU packet to addr 0x10. Alla oinitierade MU's ska lyssna p den adressen.*/
	return;
}

uint8_t muAddInitSampl(void)
{
	uint8_t sampNo = sampAlloc();
	
	sampArray[sampNo].next = sampNo;
	sampArray[sampNo].prev = sampNo;
	tmrGetHundred();
	sampArray[sampNo].timeY = tmrGetYear();
	sampArray[sampNo].timeMo = tmrGetMonth();
	sampArray[sampNo].timeD = tmrGetDay();
	sampArray[sampNo].timeH = tmrGetHour();
	sampArray[sampNo].timeM = tmrGetMinute();
	sampArray[sampNo].timeS = tmrGetSecond();
	sampArray[sampNo].value = 0;
	return sampNo;
}

void muAddSampl(uint8_t value, uint8_t addTo)
{
	uint8_t sampNo = sampAlloc();
	sampArray[sampNo].prev = sampArray[addTo].prev;
	sampArray[sampNo].next = addTo;
	sampArray[addTo].prev = sampNo;
	sampArray[sampArray[addTo].prev].next = sampNo;
	tmrGetHundred();
	sampArray[sampNo].timeY = tmrGetYear();
	sampArray[sampNo].timeMo = tmrGetMonth();
	sampArray[sampNo].timeD = tmrGetDay();
	sampArray[sampNo].timeH = tmrGetHour();
	sampArray[sampNo].timeM = tmrGetMinute();
	sampArray[sampNo].timeS = tmrGetSecond();
	sampArray[sampNo].value = value;
}

/**
* Adds a packet of Weather data into the corresponding WIT-list for a given MU.
* The MU is found in the packet itself.
*/
void muAddPacket(pack* packet)
{
	int i;
	uint8_t dataHeader, prevval;

	if(packet->ctrl == PACK_SAMP){
		uint8_t mu = muFindAddr(packet->unit);
		for(i=0; i < packet->length; i=i+2){
			/*If rain, increment by value*/
			if(packet->data[i] == RAIN){
				dataHeader = packet->data[i];
				prevval = muGetSamp(packet->unit, (dataHeader & 0xF0)>>4);
				muAddSampl(packet->data[i+1]+prevval, muList[mu].witSampl[(dataHeader & 0xF0)>>4]);
			/*If not rain, simply save value*/
			} else {
				dataHeader = packet->data[i];
				muAddSampl(packet->data[i+1], muList[mu].witSampl[(dataHeader & 0xF0)>>4]);
			}
		}
	}
}

void muSetWitAvail(uint16_t availible, uint8_t mu)
{
	muList[mu].witAvail = availible;
}

uint16_t muGetWitAvail(uint8_t mu)
{
	return muList[mu].witAvail;
}

void muSetWitEnabl(uint16_t enable, uint8_t mu)
{
	muList[mu].witEnabl = enable;
}

uint16_t muGetWitEnabl(uint8_t mu)
{
	return muList[mu].witEnabl;
}

void muSetAddr(uint8_t addr, uint8_t mu)
{
	muList[mu].address = addr;
}

uint8_t muGetAddr(uint8_t mu)
{
	return muList[mu].address;
}

uint8_t muAddUnit(void)
{
	uint8_t i;
	
	for(i=0; i<8; i++){
		if(muList[i].address == 0x10){
			muList[i].address = i+8;
			return i+8;
		}
	}
	return 0x10;
}

/**
* Returns the muList number of the mu corresponding to the address addr.
* If no mu is found, the returnvalue is 9.
*/
uint8_t muFindAddr(uint8_t addr)
{
	uint8_t i;
	for(i=0;i<8;i++){
		if(muList[i].address == addr){
			return i;
		}
	}
	return 9;
}

/**
 * Requests a sample for WIT i, from TODO:"all MU holding that WIT".
 */
void muSampleWIT(uint16_t toSample)
{ 
	uint8_t val;
	uint16_t i,j;
	
	for(i=TEMP; i<=WIND_DIR; i++){
		if(toSample & 0x01<<i){
			/*Calc MU[0]'s value*/
			val = muCalcAverage(i);
			muAddSampl(val, muList[0].witSampl[i]);
			for(j=1; j<8; j++){
				/*If mu no. j has WIT availible, sample*/
				if(muGetWitAvail(j) & 0x01<<i){
					val = muReqSamp(j, i);
					muAddSampl(val, muList[j].witSampl[i]);
				}
			}
		}
	}
	if(toSample & 0x01<<TENDENCY){
		for(j=0; j<8; j++){
			/*If mu no. j has required WITs availible, calculate tendency*/
			if(muGetWitAvail(j) & NEEDED_FOR_TENDENCY_CALC){
				val = muCalcTendency(j);	/*j == mu*/
				muAddSampl(val, muList[j].witSampl[i]);
			}
		}
	}
	if(toSample & 0x01<<HUMID_HIST){
		val = muCalcAverage(HUMID);
		muAddSampl(val, muList[0].witSampl[HUMID_HIST]);
		for(j=1; j<8; j++){
			if(muGetWitAvail(j) & 0x01<<HUMID_HIST){
				val = muReqSamp(j, HUMID);
				muAddSampl(val, muList[j].witSampl[HUMID_HIST]);
			}
		}
	}
	if(toSample & 0x01<<ATM_PRESSURE_HIST){
		val = muCalcAverage(ATM_PRESSURE);
		muAddSampl(val, muList[0].witSampl[ATM_PRESSURE_HIST]);
		for(j=1; j<8; j++){
			if(muGetWitAvail(j) & 0x01<<ATM_PRESSURE_HIST){
				val = muReqSamp(j, ATM_PRESSURE);
				muAddSampl(val, muList[j].witSampl[ATM_PRESSURE_HIST]);
			}
		}
	}
	if(toSample & 0x01<<TEMP_HIST){
		val = muCalcAverage(TEMP);
		muAddSampl(val, muList[0].witSampl[TEMP_HIST]);
		for(j=1; j<8; j++){
			if(muGetWitAvail(j) & 0x01<<TEMP_HIST){
				val = muReqSamp(j, TEMP);
				muAddSampl(val, muList[j].witSampl[TEMP_HIST]);
			}
		}
	}
	if(toSample & 0x01<<WIND_SPEED_HIST){
		val = muCalcAverage(WIND_SPEED);
		muAddSampl(val, muList[0].witSampl[WIND_SPEED_HIST]);
		for(j=1; j<8; j++){
			if(muGetWitAvail(j) & 0x01<<WIND_SPEED_HIST){
				val = muReqSamp(j, WIND_SPEED);
				muAddSampl(val, muList[j].witSampl[WIND_SPEED_HIST]);
			}
		}
	}
}

uint8_t muCalcTendency(uint8_t WIT)
{
}

uint8_t muCalcAverage(uint8_t WIT)
{
}

uint8_t muReqSamp(uint8_t mu, uint8_t WIT)
{
}


uint8_t muGetSamp(uint8_t mu, uint8_t WIT)
{
	uint8_t retval;
	sampList firstNode;
	
	firstNode = sampArray[muList[mu].witSampl[WIT]];
	retval = sampArray[firstNode.prev].value;
	
	return retval;
}