/*************************************************************************
*                            commHandler.c
*			    ---------------
*
****************************************************************************/

/**
* Includes
*/
#include "INC/stdio.h"

#include "INC/inttypes.h"
#include "INC/defines.h"

#include "INC/defrad.h"
#include "INC/r_rad.h"

#include "INC/r_com.h"
#include "INC/defcom.h"

#include "INC/r_main.h"

/**
* Private function declarations
*/
void 	comInspect(pack* pPacket);
void 	comBuildStatus(pack*);
void 	comBuildSample(pack* pInPacket,pack* pOutPacket);
void 	comBuildSetAddr(pack* pOutPacket, uint8_t setAddr);
uint8_t comAddDU();


/**
* Private variables
*/
static int i;
char ackArrived;
uint8_t myAddr;
uint8_t duList[8];
extern uint8_t* leds;

/***********************************************************************************
* Private functions
************************************************************************************/
/**
* Initiates the communication handler.
* Starts a receiver thread.
*/
void comInit()
{
	setReceiverEnabled(TRUE);
	ackArrived = FALSE;
	for(i=0; i<8; i++)
		duList[i] = 0x20;
}

/***********************************************************************************
* Public functions
************************************************************************************/
/**
* Receiver thread.
*/
void comReceive()
{
	pack receivedPacket;
	uint8_t* pBuff;
	
	pBuff = (uint8_t*)rad_getBuff();
	comPacketDecode(pBuff, &receivedPacket);
	
	if(receivedPacket.to == myAddr){
		comInspect(&receivedPacket);
	}
}

void comInspect(pack* pPacket)
{
	pack mypacket;
	uint8_t i, packStatTo = myAddr;
	
	switch(pPacket->ctrl) {
		case PACK_STATUS:
			/*If packet is for someone else, just pass on*/
			if(packStatTo != myAddr){
				pPacket->to = packStatTo;
				comSend(pPacket, packStatTo);
				packStatTo = myAddr;
			/*If packet is for me, check it out*/
			} else if(pPacket->from == 0x00){	/*egentligen 0x20, men 2'an maskas bort p vgen*/
				if(pPacket->length == 3){
					/*If packet is from unknown MU, add it*/
					if(pPacket->data[1] == TYPE_MU){
						comBuildSetAddr(&mypacket, muAddUnit());
						comSend(&mypacket, 0x10);
					/*If packet is from unknown DU, add it*/
					} else if(pPacket->data[1] == TYPE_DU){
						for(i=0; i<8; i++){
							if(duList[i] == 0x20){
								duList[i] = i;
								comBuildSetAddr(&mypacket, i+1);
								comSend(&mypacket, 0x20);
								break;
							}
						}
					}
				}
			}
			break;
		case PACK_SATUS_REQ:
			if(pPacket->unit == myAddr){
				comBuildStatus(&mypacket);
				mypacket.to = pPacket->from;
				comSend(&mypacket, pPacket->from);
			} else{
				mypacket.to = pPacket->unit;
				mypacket.from = myAddr;
				pPacket->ctrl = PACK_SATUS_REQ;
				pPacket->unit = pPacket->unit;
				pPacket->length = 0x00;
				packStatTo = pPacket->from;
				comSend(&mypacket, pPacket->unit);
			}
			break;
		case PACK_ACK:
			/*This feature is removed due to built in Auto-Ack in radio module.
			ackArrived = TRUE;*/
			break;
		case PACK_FIND_MU:
			/*Since this is a CU, not a MU, the request is interpreted as "update your MU-list"
			  If the unit would be a MU, the unit would responde with a */
			muUpdateUnitList();
			break;
		case PACK_SET_ADDR:
			if(!(pPacket->from == 0x00)){
				/*If request was from unit with address, assign new and reclaim old address*/
				for(i=0; i<8; i++){
					if(duList[i] == 0x20){
						duList[i] = i;
						comBuildSetAddr(&mypacket, i+1);
						comSend(&mypacket, pPacket->from);
						break;
					}
				}
				duList[pPacket->from] = 0x20;	/*free the old address*/
			}
			break;
		case PACK_SAMP_REQ:
			/*reply with requested samples*/
			comBuildSample(pPacket,&mypacket);
			comSend(&mypacket, pPacket->from);
			break;
		case PACK_SAMP:
			/*insert the samples into the lists*/
			muAddPacket(pPacket);
			break;
		case PACK_STOF:
			/*set the STOF flag for the specified unit*/
			muSetWitEnabl((pPacket->data), muFindAddr(pPacket->unit));
			break;
		case PACK_HTOF:
			/*/forward the HTOF command to the specified unit*/
			muSetWitAvail((pPacket->data), muFindAddr(pPacket->unit));
			break;
		case PACK_ESD:
			/*Enable the SD-card and save old samples in direcotory <date>_backlog and start sample
			  to directory <date>_log*/
			break;
		case PACK_DSD:
			/*/Disable the SD-card*/
			break;
		case PACK_SET_CU_TIME:
			/*/Set the time in the Realtime clock*/
			tmrSetYear(pPacket->data[0]);
			tmrSetMonth(pPacket->data[1]);
			tmrSetDay(pPacket->data[2]);
			tmrSetHour(pPacket->data[3]);
			tmrSetMinute(pPacket->data[4]);
			tmrSetSecond(pPacket->data[5]);
			break;
		case PACK_SAMP_INT:
			for(i=0; i<pPacket->length; i=i+2){
				setInterruptInterval(pPacket->data[(i*2)+1], pPacket->data[i*2]);
			}
			break;
		case PACK_CLEAR_HIST:
			/*/clear all the history for the specified unit*/
			muClearSamples(pPacket->data[0]);
			break;
		case PACK_SET_HIST_INT:
			/*/Set the intevall between the displayed values of the history WIT's
			   Disabled for now.*/
			
			break;
	}
}

/**
* For sending packets
*/
void comSend(pack* pPacket, uint8_t toAddr)
{
	static uint8_t reentrant = 0;
	uint8_t aPayload[32];
	
	while(reentrant);	/*OBS. Inte sker test-and-set, gr atomisk?*/
	reentrant = 1;
		pPacket->to = toAddr;
		comPacketEncode(pPacket, aPayload);
		rad_setTXAddr(toAddr);
		TX_Mode();
		TX_Data(aPayload, 32);
		RX_Mode();
	reentrant = 0;
	debug(4);
}

/**
* packetEncode encodes a pack-struct to a byte array that can be sent by the radio
*/
void comPacketEncode(pack* pPacket, uint8_t* payload)
{
	*payload = (((pPacket->to)<<4 & UPPER) | (pPacket->from & LOWER));
	*(payload+1) = (((pPacket->ctrl)<<4 & UPPER) | (pPacket->unit & LOWER));
	*(payload+2) = pPacket->length;
	
	for(i=0; i<pPacket->length; i++){
		/*byte at position pPacket->data+i*/
		*(payload+3+i) = pPacket->data[i];
	}
}

/**
* packetEncode encodes a pack-struct to a byte array that can be sent by the radio
*/
void comPacketDecode(uint8_t* pBytes, pack* pOutPacket)
{
	uint8_t length;
	
	length = *(pBytes+2);
	pOutPacket->to = (*pBytes & UPPER)>>4;
	pOutPacket->from = (*pBytes & LOWER);
	pOutPacket->ctrl = (*(pBytes+1)&UPPER)>>4;
	pOutPacket->unit = *(pBytes+1) & LOWER;
	pOutPacket->length = length;
	for(i=0; i<length; i++){
		pOutPacket->data[i] = *(pBytes+3+i);
	}
}

/**
* Builds a status packet to respond with when asked for status.
*/
void comBuildStatus(pack* pPacket)
{
	/*Sample clock*/
	tmrGetHundred();
	pPacket->to = 0x00;
	pPacket->from = myAddr;
	pPacket->ctrl = PACK_STATUS;
	pPacket->unit = 0x00;
	pPacket->length = 0x09;
	pPacket->data[0] = 0x00;
	pPacket->data[1] = TYPE_CU;
	pPacket->data[2] = muGetWitEnabl(0x00);
	pPacket->data[3] = tmrGetYear();
	pPacket->data[4] = tmrGetMonth();
	pPacket->data[5] = tmrGetDay();
	pPacket->data[6] = tmrGetHour();
	pPacket->data[7] = tmrGetMinute();
	pPacket->data[8] = tmrGetSecond();
}

/**
* Builds a sample packet to respond with when asked for sample.
*/
void comBuildSample(pack* pInPacket,pack* pOutPacket)
{
	unsigned int i;
	static uint8_t val = 20;
	pOutPacket->to = pInPacket->from;
	pOutPacket->from = myAddr;
	pOutPacket->ctrl = PACK_SAMP;
	pOutPacket->unit = pInPacket->unit;
	pOutPacket->length = (pInPacket->length)+(pInPacket->length);	/*Add bttre n mult eftersom vi kr utan optimering*/
/*/ Take a value from an array with the index being a const int that is incremented*/
	for(i=0; i<(pInPacket->length); i++){
		pOutPacket->data[i*2] = pInPacket->data[i];
		pOutPacket->data[i*2+1] = val++;/*muGetSamp(pInPacket->unit, pInPacket->data[i]);*/
	}
	if(val>30)
		val = 15;
}

void comBuildSetAddr(pack* pOutPacket, uint8_t setAddr)
{
	pOutPacket->to = setAddr;
	pOutPacket->from = myAddr;
	pOutPacket->ctrl = PACK_SET_ADDR;
	pOutPacket->unit = setAddr;
	pOutPacket->length = 1;
	pOutPacket->data[0] = setAddr;
}

uint8_t comAddDU(void)
{	
	for(i=0; i<8; i++){
		if(duList[i] == 0x20){
			duList[i] = i;
			return i;
		}
	}
	return 0x20;
}

void comTest()
{
	pack statuspack;
	comBuildStatus(&statuspack);
	comSend(&statuspack, 0x01);
}

