/******************************************************************************
*                            radio.c
*                            ------
*   Detta r radio-modulen. Det r ett hjlpobjekt till comunication handlern.
*******************************************************************************/

/**
* Includes
*/
#include "INC/stdio.h"
#include "INC/inttypes.h"
#include "INC/defines.h"

#include "INC/defrad.h"
#include "INC/r_rad.h"

#include "INC/r_spi.h"
#include "INC/defspi.h"

/*only needed for rad_test()*
#include "lcd_prot.h"
#include "lcd_def.h"*/


/**
* Private function prototypes
*/

/**
* Variables
*/
char* led = (char*)0x3C001C;
extern uint8_t IRQ_Source;
uint8_t nRF_Buffer[32];
extern char receiverEnabled;
extern uint8_t myAddr;

/**
* Constants
*/
uint8_t TX_ADDRESS[TX_ADR_LENGTH]  = {0x69,0x50,0xc1,0x7e,0xd1};


/***********************************************************************************
* Public functions
************************************************************************************/
/*ISR(PCINT1_vect)
{*/
void rad_IRQ(void)
{
	uint8_t temp,rx_pw,tmpval, pipe;
	
	/*TODO: cli();*/
	temp = rad_getstat();
	if(temp & RX_DR){
		while(!(rad_getfifo() & 0x01)){
			IRQ_Source = RX_DR;
			tmpval = RX_DR;
			rad_putreg(&tmpval, STATUS, 1);
			
			pipe = ((temp>>1)&0x07);
			rad_getreg(&rx_pw, RX_PW_P0 + pipe, 1);
			RX_Data(nRF_Buffer, rx_pw);
			
			comReceive();
		}
	}
	if(temp & TX_DS){
		IRQ_Source = TX_DS;
		tmpval = TX_DS;
		rad_putreg(&tmpval, STATUS, 1);
		if(rad_getfifo() & 0x10)
			RX_Mode();
		spiSetCE(1);
	}
	if(temp & MAX_RT){
		IRQ_Source = MAX_RT;
		tmpval = MAX_RT;
		rad_putreg(&tmpval, STATUS, 1);
		rad_clean();
		RX_Mode();
		spiSetCE(1);
	}
	if(temp & TX_FULL){
		rad_clean();
	}
	
	/*TODO: sei();*/
}

void rad_init(void)
{
	int i;
	uint8_t d,tmp[5];
	
	myAddr = 0x00;
	
	spiSetCE(0);
	
	d=3;
	rad_putreg(&d, SETUP_AW, 1);
	
	for(i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = myAddr;
	rad_putreg(tmp, RX_ADDR_P1, TX_ADR_LENGTH);
	
	for(i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = 0x00;
	rad_putreg(tmp, TX_ADDR, TX_ADR_LENGTH);
	for(i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = 0x00;
	rad_putreg(tmp, RX_ADDR_P0, TX_ADR_LENGTH);
	
	d=0x03;
	rad_putreg(&d, EN_RXADDR, 1);

	d=0x03;
	rad_putreg(&d, EN_AA, 1);
	
	d = 0x0f;	/*0x0f fr retransmit*/
	rad_putreg(&d, SETUP_RETR, 1);

	d=0x29;
	rad_putreg(&d, RF_CH, 1);
	d=0x07;
	rad_putreg(&d, RF_SETUP, 1);
	
	d=TX_PLOAD_WIDTH;
	rad_putreg(&d, RX_PW_P0, 1);
	d=TX_PLOAD_WIDTH;
	rad_putreg(&d, RX_PW_P1, 1);

	d=0x0f;
	rad_putreg(&d, CONFIG, 1);
}


void rad_clean(void)
{
	uint8_t fifostat;
	
	rad_getreg(&fifostat, FIFO_STATUS, 1);
	if(!(fifostat & 0x10)){
		rad_cmd(FLUSH_TX);
	}
	if(!(fifostat & 0x01)){
		/*RX_fifo not empty*/
		if(IRQ_Source == IDLE){
			rad_cmd(FLUSH_RX);
		}
	}
}

uint8_t* rad_getBuff()
{
	return nRF_Buffer;
}

void rad_setTXAddr(uint8_t toAddr)
{
	uint8_t tmp[5],i;
	for(i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = toAddr;
	rad_putreg(tmp, TX_ADDR, TX_ADR_LENGTH);
	
	for(i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = toAddr;
	rad_putreg(tmp, RX_ADDR_P0, TX_ADR_LENGTH);
}

/**
* Send data on initialized nRF24l01
* One high pulse(>10s) on CE will now send this
* packet and expext an acknowledgment from the RX device.
*/
void TX_Data(uint8_t* pPayload, uint8_t bytes)
{
	uint8_t cmd,temp, i;
	spiSetCE(0);
	
	/*_delay_ms(5);*
	for(temp=0;temp<255;temp++);*/
	
	cmd = W_TX_PAYLOAD;
	spiStart(SPI_RADIO);
	spiTransMulti(&cmd,1);
	spiTransMulti(pPayload, bytes);
	spiEnd(SPI_RADIO);

	spiSetCE(1);
	/*_delay_ms(50);*/
	for(temp=0;temp<255;temp++)
		for(i=0; i<70; i++);
}

/**
* Retrieve data from initialized nRF24l01
*/
void RX_Data(uint8_t* pPayload, uint8_t bytes)
{
	uint8_t temp,i;
	uint8_t cmd;
	spiSetCE(0);
	cmd = R_RX_PAYLOAD;
	spiStart(SPI_RADIO);
	spiTransMulti(&cmd,1);
	spiTransMulti(pPayload, bytes);
	spiEnd(SPI_RADIO);
	/*for(temp=0;temp<255;temp++)
		for(i=0; i<50; i++);*/
}

/**
* This function initializes one nRF24L01 device to
* TX mode, set TX address, select RF channel, datarate & TX pwr.
* PWR_UP is set, CRC(2 bytes) is enabled, & PRIM:TX.
*/
void TX_Mode(void)
{
	uint8_t d;
	spiSetCE(0);
	d = 0x0e;
	rad_putreg(&d, CONFIG, 1);
}


/**
* Starts the receiveing process. When an irq occurrs, do a rad_rx() to retreive the data.
* To power down the radio, run rad_pwrDwn().
*/
void RX_Mode(void)
{
	uint8_t d;
	spiSetCE(0);
	d=0x0f;
	rad_putreg(&d, CONFIG, 1);
}

/**
* Puts the radio in power down mode.
*/
void rad_stdbymode(uint8_t mode)
{
	uint8_t conf;
	switch(mode){
		case 0:
			/*Total power down, time to power up: 150us*/
			rad_getreg(&conf, CONFIG, 1);
			conf = conf & 0xFD;
			rad_putreg(&conf, CONFIG, 1);
			break;
		case 1:
			/*Standby mode I, time to RX/TX: 130us*/
			rad_getreg(&conf, CONFIG, 1);
			conf = conf | 0x02;
			rad_putreg(&conf, CONFIG, 1);
			spiSetCE(0);
			break;
		case 2:
		default:
			/* Standby mode II - "Not in standby at all, just nothing to send.",
			   time to TX: 130us */
			rad_getreg(&conf, CONFIG, 1);
			conf = conf | 0x02;
			rad_putreg(&conf, CONFIG, 1);
			spiSetCE(1);
			break;	
	}
}

/**
* Function to read the status register of the radio.
*/
uint8_t rad_getstat() 
{
	uint8_t status = 0xFF;
	spiStart(SPI_RADIO);
	spiTransMulti(&status,1);
	spiEnd(SPI_RADIO);
	return status;

}
/***********************************************************************************
* Private functions
************************************************************************************/
/**
* Function to put one or more byte(s) in a register of the radio.
*/
uint8_t rad_putreg(uint8_t* pData, uint8_t reg, uint8_t bytes)
{	
	uint8_t status;
	uint8_t regis = W_REGISTER(reg);

	spiStart(SPI_RADIO);
	status = spiTransieve(regis);
	spiTransMulti(pData, bytes);
	spiEnd(SPI_RADIO);

	return(status);
}

/**
* Function to read one or more byte(s) from a radio register.
*/
uint8_t rad_getreg(uint8_t* pBuff, uint8_t reg, uint8_t bytes)
{
	uint8_t status=0;
	uint8_t regis = R_REGISTER(reg);
	
	spiStart(SPI_RADIO);
	status = spiTransieve(regis);
	spiTransMulti(pBuff, bytes);
	spiEnd(SPI_RADIO);

	return(status);
}

uint8_t rad_cmd(uint8_t to_flush)
{	
	uint8_t status;
	spiStart(SPI_RADIO);
	status = spiTransieve(to_flush);
	spiEnd(SPI_RADIO);

	return(status);
}

/***********************************************************************************
* Testcases
************************************************************************************/
void rad_test(void)
{
}

void rad_showconf()
{
	char temp = 0;
	
	rad_getreg(&temp, CONFIG, 1);  
	*led = temp;
	debug(2);
}
void rad_showstat()
{
	char temp = 0;
	
	rad_getreg(&temp, STATUS, 1);  
	*led = temp;
	debug(2);
}
void rad_showfifo()
{
	char temp = 0;
	
	rad_getreg(&temp, FIFO_STATUS, 1);  
	*led = temp;
	debug(2);
}
uint8_t rad_getfifo(void)
{
	char temp = 0;
	
	rad_getreg(&temp, FIFO_STATUS, 1);  
	return temp;
}
