/*************************************************************************
*                            spi.c
*                            ------
*   Detta r modulen som skter SPI-interfacet.
****************************************************************************/

/**
* Includes
*/
#include "INC/stdio.h"

#include "INC/inttypes.h"
#include "INC/defines.h"
#include "INC/defspi.h"



/**
* Private function declarations
*/

/**
* Private variables
*/
char out;
char in;

/***********************************************************************************
* Private functions
************************************************************************************/

/***********************************************************************************
* Public functions
************************************************************************************/
void spiSetCE(uint8_t val)
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;
	
	if(val == 0){
		out &= ~SPI_CE_RAD;
	} else {
		out |= SPI_CE_RAD;
	}
	*SPI = out;
}
void spiStart(uint8_t SPI_CS_PIN)
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;

	out &= ~SPI_CS_PIN;					/*stter enhetens csn till noll*/
	*SPI = out;
}
void spiEnd(uint8_t SPI_CS_PIN)
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;

	out |= SPI_CS_PIN;					/*stter enhetens csn till ett*/
	*SPI = out;
}

/**
* Initiates the spi interface
*/
void spiInit()
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;
	
	out = SPI_RADIO | SPI_SDC;		/*SPI_CS_PIN r aktiv lg, SPI_TMR r aktiv lg och spi-klockan r aktiv lg*/	
	in=*SPI;
	*SPI = out;					/*Lgg ut p d-vippor*/
}


/**
* Function to exchange bytes with a spi perefiral.
*/
uint8_t spiTransieve(uint8_t payload)
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;

	uint8_t response=0,a;
	int i;
	
	for(i=7;i>=0;i--){
		if(payload & (0x01<<i)){			/*om frsta biten (MSB) r etta ska en etta skickas...*/
			out |= SPI_MOSI;			/*lgg mosi-bit som etta*/
		} else{
			out &= ~SPI_MOSI;			/*lgg mosi-bit som nolla*/
		}
		*SPI = out;
		
		out |= SPI_CLK;			 		/*klocka in*/
		*SPI = out;
		
		for(a=0;a<50;a++);	/*delay*/
		
		in = *SPI;
		response |= (in & SPI_MISO)<<i;
		
		out &= ~SPI_CLK;					/*slpp klocka*/
		*SPI = out;
		for(a=0;a<50;a++);	/*delay*/
	}
	return response;
}

/**
* Function to exchange multiple bytes with a spi perefiral.
* Returned data is found in payload.
*/
void spiTransMulti(uint8_t* payload, uint8_t nrOfBytes)
{
	unsigned short int *SPI = (unsigned short int *) 0x0040000;
	int j,i;
	
	
	for(j=0;j<nrOfBytes;j++){
		for(i=7;i>=0;i--){
			if(*(payload+j) & (1<<i)){			/*om frsta biten (MSB) r etta ska en etta skickas...*/
				out |= SPI_MOSI;			/*lgg mosi-bit som etta*/
			} else{
				out &= ~SPI_MOSI;			/*lgg mosi-bit som nolla*/
			}
			*SPI = out;
			
			out |= SPI_CLK;			 		/*klocka in*/
			*SPI = out;
			
			/*for(a=0;a<50;a++);	/*delay*/
		
			in = *SPI;
			if(in & SPI_MISO)
				*(payload+j) |= 0x01<<i;
			else
				*(payload+j) &= ~(0x01<<i);
				
			out &= ~SPI_CLK;				/*slpp klocka*/
			*SPI = out;
			/*for(a=0;a<50;a++);	/*delay*/
		}
	}	
}
