/*************************************************************************
*                            commHandler.c
*			    ---------------
*
****************************************************************************/

/**
* Includes
*/
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include "defines.h"
#include "commHandler_def.h"
#include "commHandler_prot.h"
#include "radio_def.h"
#include "radio_prot.h"

#include "spi_def.h"
#include "lcd_prot.h"
#include "lcd_def.h"
#include "du_def.h"

/**
* Private function declarations
*/

/**
* Variables
*/
static uint8_t myAddr;
static uint8_t currWITVal[11];


/***********************************************************************************
* Private functions
************************************************************************************/
/**
* Initiates the communication handler.
* Starts a receiver thread.
*/
void comInit()
{
	/*Addr 0x00 is for CU, 0x01-0x07 is for DU's and 0x08-0xff is for MU's*/
}

/***********************************************************************************
* Public functions
************************************************************************************/
/**
* Receiver thread.
*/
void comReceive()
{
	pack receivedPacket;
	uint8_t* pBuff;
	
	pBuff = rad_getbuff();
	comPacketDecode(pBuff, &receivedPacket);
	comInspect(&receivedPacket);
}

void comInspect(pack* pPacket)
{
	pack mypacket;
	switch(pPacket->ctrl) {
		case PACK_STATUS:
			/*TODO:*/
			break;
		case PACK_STATUS_REQ:
			/*/reply with status*/
			comBuildStatus(&mypacket);
			mypacket.to = pPacket->from;
			comSend(&mypacket, (char)pPacket->from);
			break;
		case PACK_ACK:
			/*This feature is removed due to built in Auto-Ack in radio module.*/
			break;
		case PACK_FIND_MU:
			/*ignore, this is not a MU or CU*/
			break;
		case PACK_SET_ADDR:
			myAddr = pPacket->data[0];
			rad_setRXAddr(myAddr);
			lcd_clearscrn();
			lcd_setxy(5,2);
			lcd_printstr("Address changed");
			lcd_setxy(25,3);
			lcd_printstr("to ");
			lcd_printhex(myAddr);
			_delay_ms(1000);
			setReceiverEnabled(1);
			break;
		case PACK_SAMP_REQ:
			/*Ignore on DU*/
			break;
		case PACK_SAMP:
			/*Update values on screen (eg. in variables holding screen-values)*/
				putvalues(pPacket->unit, pPacket->data[0],pPacket->data[1]);
				lcd_setxy(0,0);
				lcd_printstr("Samp in, WIT: ");
				lcd_printhex(pPacket->data[0]);
				_delay_ms(50);
			//}
			updateUI();
			break;
		case PACK_STOF:
			/*Ignore on DU*/
			break;
		case PACK_HTOF:
			/*Ignore on DU*/
			break;
		case PACK_ESD:
			/*Ignore on DU*/
			break;
		case PACK_DSD:
			/*Ignore on DU*/
			break;
		case PACK_SET_CU_TIME:
			/*Ignore on DU*/
			break;
		case PACK_SAMP_INT:
			/*Ignore on DU*/
			break;
		case PACK_CLEAR_HIST:
			/*Ignore on DU*/
			break;
		case PACK_SET_HIST_INT:
			/*Removed*/
			break;
	}
}

/**
* For sending packets
*/
void comSend(pack* pPacket, uint8_t toAddr)
{
	static uint8_t reentrant = 0;
	uint8_t aPayload[32];
	
	if(!(rad_getstat() & 0x01)){
		while(reentrant);	/*OBS. Inte sker test-and-set, gr atomisk?*/
		reentrant = 1;
			pPacket->to = toAddr;
			comPacketEncode(pPacket, aPayload);
			rad_setTXAddr(toAddr);
			TX_Mode();
			TX_Data(aPayload, 32);
			RX_Mode();
		reentrant = 0;
	}
}

/**
* packetEncode encodes a pack-struct to a byte array that can be sent by the radio
*/
void comPacketEncode(pack* pPacket, uint8_t* payload)
{
	*payload = (((pPacket->to)<<4 & UPPER) | (pPacket->from & LOWER));
	*(payload+1) = (((pPacket->ctrl)<<4 & UPPER) | (pPacket->unit & LOWER));
	*(payload+2) = pPacket->length;
	
	for(uint8_t i=0; i<pPacket->length; i++){
		/*byte at position pPacket->data+i*/
		*(payload+3+i) = pPacket->data[i];
	}
}

/**
* packetEncode encodes a pack-struct to a byte array that can be sent by the radio
*/
void comPacketDecode(uint8_t* pBytes, pack* pOutPacket)
{
	uint8_t length;
	
	length = *(pBytes+2);
	pOutPacket->to = (*pBytes & UPPER)>>4;
	pOutPacket->from = (*pBytes & LOWER);
	pOutPacket->ctrl = (*(pBytes+1)&UPPER)>>4;
	pOutPacket->unit = *(pBytes+1) & LOWER;
	pOutPacket->length = length;
	for(uint8_t i=0; i<length; i++){
		pOutPacket->data[i] = *(pBytes+3+i);
	}
}

/**
* Builds a status packet to respond with when asked for status.
*/
void comBuildStatus(pack* pPacket)
{
	/*Sample clock*/
	pPacket->to = 0x00;
	pPacket->from = myAddr;
	pPacket->ctrl = PACK_STATUS;
	pPacket->unit = 0x00;
	pPacket->length = 0x03;
	pPacket->data[0] = 0x00;
	pPacket->data[1] = TYPE_CU;
	pPacket->data[2] = 0x00;		///muGetWitEnabl(0x00);
}

/**
* Builds a status request packet.
*/
void comBuildStatusReq(pack* pPacket)
{
	/*Sample clock*/
	pPacket->to = 0x00;
	pPacket->from = myAddr;
	pPacket->ctrl = PACK_STATUS_REQ;
	pPacket->unit = 0x00;
	pPacket->length = 0x00;
}

/**
* Builds a status request packet.
*/
void comBuildSampReq(pack* pPacket, uint8_t field_mu, uint8_t field_wit)
{
	/*Sample clock*/
	pPacket->to = 0x00;
	pPacket->from = myAddr;
	pPacket->ctrl = PACK_SAMP_REQ;
	pPacket->unit = field_mu;
	pPacket->length = 1;
	pPacket->data[0] = field_wit;
}

/**
* Builds a status request packet.
*/
void comBuildAddrReq(pack* pPacket)
{
	/*Sample clock*/
	pPacket->to = 0x00;
	pPacket->from = myAddr;
	pPacket->ctrl = PACK_SET_ADDR;
	pPacket->unit = myAddr;
	pPacket->length = 0x00;
}

/**
* Builds a sample packet to respond with when asked for sample.
*/
void comBuildSample(pack* pInPacket,pack* pOutPacket)
{
/// Take a value from an array with the index being a const int that is incremented
}

uint8_t getMyAddr(void)
{
	return myAddr;
}

void setMyAddr(uint8_t addr)
{
	myAddr = addr;
}

uint8_t getValues(uint8_t WIT)
{
	return currWITVal[WIT];
}

void comTest()
{
	pack sendpack;
	comBuildStatusReq(&sendpack);
	comSend(&sendpack, 0x00);
}

