/******************************************************************************
*                            du.c
*                            ----
*   Detta är Display Unit koden.
*******************************************************************************/

/**
* Includes
*/
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <string.h>
#include "defines.h"
#include "du_def.h"

#include "spi_def.h"
#include "spi_prot.h"
#include "lcd_def.h"
#include "lcd_prot.h"
#include "radio_def.h"
#include "radio_prot.h"
#include "commHandler_def.h"
#include "commHandler_prot.h"


/**
* Private function prototypes
*/
void ritaruta(void);
void ritadata(void);
void init(void);
uint8_t checkbuttons(void);
void navigate(uint8_t pressed);
void buildmenutree(void);
void addmenupos(menu_t* pMenuposParent, uint8_t name);
void setfieldwit(menu_t* pMenupos);
void getaddr(menu_t*);
void clean(menu_t* pMenupos);
void printroot(menu_t* mp);
int childcount(menu_t* mp);
void printmenu(menu_t* mp);
void disunit(menu_t* pMenupos);
void testradio(menu_t* pMenupos);
void setmask(int mu);

/**
* Variables
*/
uint8_t latestdata[10];	//one for each WIT
menu_t* menupos;
extern uint8_t nRF_Buffer[32];
extern const char stdfont[505];
extern const unsigned char solmoln_bits[];
extern const unsigned char regnmoln_bits[];
extern const unsigned char moln_under_bits[];
extern const unsigned char moln_over_bits[];
extern const unsigned char sol_bits[];
extern uint8_t myAddr;

static dispfield_t fields[4];



#define NAME_MOTOROLA		0
#define NAME_MAIN_MENU		9
#define NAME_DISPLAY		19
#define NAME_FIELD_1		27
#define NAME_FIELD_2		35
#define NAME_FIELD_3		43
#define NAME_FIELD_4		51
#define NAME_CHOSE_WIT		59
#define NAME_CHOSE_MU		69
#define NAME_UNIT_N_COM		78
#define NAME_RADIO_STATUS	89
#define NAME_TEST_RADIO		106
#define NAME_CHOSE_TEMP		117
#define NAME_CHOSE_HUMID	122
#define NAME_CHOSE_ATM		131
#define NAME_CHOSE_RAIN		138
#define NAME_CHOSE_WIND		143
#define NAME_CHOSE_WINDDIR	154
#define NAME_CHOSE_TENDENCY	169
#define NAME_CHOSE_HUMID_HIST	178
#define NAME_CHOSE_ATM_HIST	193
#define NAME_CHOSE_TEMP_HIST	207
#define NAME_CHOSE_WIND_HIST	218
#define NAME_GET_ADDR		235
#define NAME_CLEAN		247
const char menu_names[] PROGMEM = {"MotorOla\0Main menu\0Display\0Field 1\0Field 2\0Field 3\0Field 4\0Chose WIT\0Chose MU\0Unit & Com\0Radio status\0   \0Test radio\0Temp\0Humidity\0Atm.pr\0Rain\0Wind speed\0Wind direction\0Tendency\0Humidity hist.\0Atm.pr. hist.\0Temp hist.\0Wind speed hist.\0Get address\0Clean radio"};


/***********************************************************************************
* Private functions
************************************************************************************/
int main(void)
{
	cli();
	uint8_t pressed=0;

	init();
	_delay_ms(100);
	lcd_init();
	lcd_clearscrn();
	
	spi_init();
	rad_init();
	
	comInit();
	
	buildmenutree();
	(*(menupos->doit))(NULL);
	
	while(1){
		pressed = checkbuttons();
		navigate(pressed);
	}
}

ISR(TIMER0_OVF_vect)
{
	static uint8_t prescaler=0;
	pack sendpack;
	rad_IRQ();
	if(getReceiverEnabled()){
		prescaler++;

		if(prescaler == 0x0f){
			comBuildSampReq(&sendpack,fields[0].mu,fields[0].wit);//, field_mu[0], field_wit[0]);
			comSend(&sendpack, 0x00);
		} else if(prescaler == 0x1f){
			comBuildSampReq(&sendpack,fields[1].mu,fields[1].wit);// field_mu[1], field_wit[1]);
			comSend(&sendpack, 0x00);
		} else if(prescaler == 0x2f){
			comBuildSampReq(&sendpack,fields[2].mu,fields[2].wit);// field_mu[1], field_wit[1]);
			comSend(&sendpack, 0x00);
		} else if(prescaler == 0x3f){
			comBuildSampReq(&sendpack,fields[3].mu,fields[3].wit);// field_mu[1], field_wit[1]);
			comSend(&sendpack, 0x00);
			prescaler = 0;
		}
	}
}

void updateUI(void)
{
	(*(menupos->doit))(NULL);
}
void init(void)
{
	fields[0].mu=0;
	fields[0].wit = HUMID;

	fields[1].mu=0;
	fields[1].wit = WIND_SPEED;

	fields[2].mu=0;
	fields[2].wit = ATM_PRESSURE;
	
	fields[3].mu=0;
	fields[3].wit = TEMP;
	
	
	DDRB &= ~_BV(BUT0); 
	DDRB &= ~_BV(BUT1);
	DDRB &= ~_BV(BUT2);
	PORTB |= _BV(BUT0);
	PORTB |= _BV(BUT1);
	PORTB |= _BV(BUT2);
	
	
	TCCR0A = 0x00;
	TCCR0B = 0x05;
	TIMSK0 = 0x01;
	sei();
}

void navigate(uint8_t pressed)
{
	int i=1;
	switch (pressed){
		case 1:
			if(menupos->selbut > 0){
				menupos->selbut = (menupos->selbut)-1;
			}
			(*(menupos->doit))(menupos);
			break;
		case 2:
			menupos= (menupos->button[(menupos->selbut)]);
			(*(menupos->doit))(menupos);
			break;
		case 3:
			i = childcount(menupos);
			if(menupos->selbut < (i-1)){
				menupos->selbut++;
			}
			(*(menupos->doit))(menupos);
	}
}

uint8_t checkbuttons(void)
{
	uint8_t port;
	port=PINB;

	if(!(port & _BV(BUT0))){
		_delay_ms(10);
		while(!(port & _BV(BUT0))){
			port=PINB;
		}
		return 1;
	} else if(!(port & _BV(BUT1))){
		_delay_ms(10);
		while(!(port & _BV(BUT1))){
			port=PINB;
		}
		return 2;
	} else if(!(port & _BV(BUT2))){
		_delay_ms(10);
		while(!(port & _BV(BUT2))){
			port=PINB;
		}
		return 3;
	} else{
		return 0;
	}
}

void buildmenutree(void)
{
	menupos = malloc(sizeof(menu_t));
	menupos->name = NAME_MOTOROLA;
	menupos->selbut = 0;
	menupos->doit = &printroot;
	
	/*menupos is currently the only node and is the root-screen showing WIT's.*/
	menupos->button[0] = menupos;			//see 2 following lines, just so that addmenupos behaves below!
	addmenupos(menupos, NAME_MAIN_MENU); 		//currently called menupos->button[1]
	menupos->button[0] = menupos->button[1]; 	//Allways go to menu when pressing select in root!
//	menupos->button[1] = NULL;
	/*
	About: Display menu

	Which sample-types to display and wich measuring-units/calculated values to take it from.
	All types can be taken from a specified mu or an average of several (chosen from a list of availible).
	*/
	addmenupos(menupos->button[1], NAME_DISPLAY);
	addmenupos(menupos->button[1]->button[1], NAME_FIELD_1);
	addmenupos(menupos->button[1]->button[1], NAME_FIELD_2);
	addmenupos(menupos->button[1]->button[1], NAME_FIELD_3);
	addmenupos(menupos->button[1]->button[1], NAME_FIELD_4);
/*	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_TEMP);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_HUMID);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_ATM);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_ATM_HIST);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_RAIN);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_WIND);
	addmenupos(menupos->button[1]->button[1]->button[1], NAME_CHOSE_WINDDIR);	
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_TEMP);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_HUMID);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_ATM);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_ATM_HIST);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_RAIN);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_WIND);
	addmenupos(menupos->button[1]->button[1]->button[2], NAME_CHOSE_WINDDIR);
	
	menupos->button[1]->button[1]->button[1]->button[1]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[2]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[3]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[4]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[5]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[6]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[1]->button[7]->doit = &setfieldwit;
/*	menupos->button[1]->button[1]->button[2]->button[1]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[2]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[3]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[4]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[5]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[6]->doit = &setfieldwit;
	menupos->button[1]->button[1]->button[2]->button[7]->doit = &setfieldwit;*/

/*
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Temp");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Humid");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Atm-pr.");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Atm-log");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Rain");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Wind");
	addmenupos(menupos->button[1]->button[1]->button[2], "Chose Wind-dir");

	/*
	About: Units menu
	
	Handle the units. Wich units is there? What can they do?
	Allso handle communication. TTL and adresses and alike.
	*/
	addmenupos(menupos->button[1], NAME_UNIT_N_COM);
	addmenupos(menupos->button[1]->button[2], NAME_RADIO_STATUS);
	addmenupos(menupos->button[1]->button[2], NAME_GET_ADDR);
	addmenupos(menupos->button[1]->button[2], NAME_CLEAN);
	
	menupos->button[1]->button[2]->button[1]->doit = &testradio;
	menupos->button[1]->button[2]->button[1]->selbut = 1;

	menupos->button[1]->button[2]->button[2]->doit = &getaddr;
	menupos->button[1]->button[2]->button[2]->selbut = 1;
	
	menupos->button[1]->button[2]->button[3]->doit = &clean;
	menupos->button[1]->button[2]->button[3]->selbut = 1;
/*
	addmenupos(menupos->button[2], "Search for units");	//If motorOla present, tell it to search for units,
								//otherwise, search for motorOla.
	addmenupos(menupos->button[1], "Disable units");	//Ask motorOla wich units there are and tell wich to disable.
	menupos->button[3]->button[1]->doit = &disunit;

	addmenupos(menupos->button[1], "Test radio");
	menupos->button[1]->button[1]->doit = &testradio;/**/
}

/*
	function: addmenupos
	
	Adds a menupos to the supplied menupos mp.
	Sets the name and initiates the doit function pointer to printmenu.
*/
void addmenupos(menu_t* pMenuposParent, uint8_t name)
{
	int i=0;
	i = childcount(pMenuposParent);
	pMenuposParent->button[i] = malloc(sizeof(menu_t));
	pMenuposParent->button[i]->button[0]= pMenuposParent;
	pMenuposParent->button[i]->name = name;
	pMenuposParent->button[i]->selbut = 0;
	pMenuposParent->button[i]->doit = &printmenu;
}

/*
	function: childcount
	
	Counts the number of childs a menu_t node has.
*/
int childcount(menu_t* pMenuposParent)
{

	int i=0;
	while(i<10){
		if(pMenuposParent->button[i] == NULL){
			return i;
		} else {
			i++;
		}
	}
	return i;
}

void setfieldwit(menu_t* pMenupos)
{
	uint8_t fieldnr = pMenupos->button[0]->button[0]->selbut;
	uint8_t wit, wit_type = pMenupos->button[0]->selbut;	//OBS. ordning ej enl. protokoll
	
	switch(wit_type){					//Därav omorganiseringen...
		case 1:	
			wit=TEMP;
			break;
		case 2:	
			wit=HUMID;
			break;
		case 3:	
			wit=ATM_PRESSURE;
			break;
		case 4:	
			wit=ATM_PRESSURE_HIST;
			break;
		case 5:	
			wit=RAIN;
			break;
		case 6:
			wit=WIND_SPEED;
			break;
		case 7:
			wit=WIND_DIR;
			break;
		case 0:
		default:
			return;
	}
	fields[fieldnr-1].wit = wit;
	menupos = menupos->button[0];
	
	lcd_clearscrn();
	lcd_setxy(38,2);
	lcd_printstr("Done!");
	_delay_ms(1000);
	(*(menupos->doit))(menupos);
}

void getaddr(menu_t* pMenupos)
{
	pack reqpack;
	comBuildAddrReq(&reqpack);
	comSend(&reqpack,0x00);
	
	menupos = menupos->button[0];
	lcd_clearscrn();
	(*(menupos->doit))(menupos);
	
/*	uint8_t pressed = 0;
	lcd_clearscrn();
	lcd_setxy(0,0);
	lcd_printstr("Display Unit address:");
	
	while(pressed != 2){
		pressed = checkbuttons();
		switch (pressed){
			case 1:
				if(menupos->selbut > 1){
					menupos->selbut = (menupos->selbut)-1;
				}
				break;
			case 2:
				menupos= (menupos->button[0]);
				break;
			case 3:
				if(menupos->selbut < (0x07)){
					menupos->selbut++;
				}
		}
		setMyAddr(menupos->selbut);
		lcd_setxy(0,1);
		lcd_printhex(getMyAddr());
	}
	(*(menupos->doit))(menupos);*/
}

void disunit(menu_t* pMenupos)
{
///TODO
}
void clean(menu_t* pMenupos)
{
	menupos = menupos->button[0];
	rad_init(); //rad_clean();
}

void testradio(menu_t* pMenupos)
{
	menupos->selbut = 0;
	
	lcd_clearscrn();
	lcd_setxy(0,0);
	lcd_printstr("Config: ");
	lcd_printhex(rad_getconf());

	lcd_setxy(0,1);
	lcd_printstr("Status: ");
	lcd_printhex(rad_getstat());

	lcd_setxy(0,2);
	lcd_printstr("fifo-status: ");
	lcd_printhex(rad_getfifostat());

	lcd_setxy(0,3);
	lcd_printstr("OBS_TX: ");
	lcd_printhex(rad_getobs());
	
	lcd_setxy(0,4);
	lcd_printstr("Status changes:");
	lcd_printhex(getSinceLastFlags());
	setSinceLastFlags(0);
}

/**
* Function: setmask
* Menu where a checkbox is showed for every measure-type
* is displayed. On the bottom there is a OK-button wich
* takes the bitmask and sends it to Ola.
*//*
void setmask(int mu)
{
///TODO
}*/



/*
	function: printmenu

	Prints a menu by printing one button for each child, with the childs
	name as text on the button and selected according to selbut in pMenupos.
*/
void printmenu(menu_t* pMenupos)
{
	int btncnt;
	int y=0;

	lcd_clearscrn();
	//if(pMenupos==NULL)
	//	pMenupos=menupos;
	btncnt = childcount(pMenupos);
	if(btncnt>7){
		if(pMenupos->selbut > 7)
			y=pMenupos->selbut - 7;
		else
			y = 0;
	}
	for(int i=0; i<8; i++){
		if(pMenupos->button[y+i] == NULL)
			break;
		lcd_setxy(0,i);
		if(y == 0 && i == 0){
			lcd_button("Back", pMenupos->selbut == 0);
		} else {
			lcd_button_P((char*)(menu_names+(pMenupos->button[y+i]->name)), (pMenupos->selbut == (y+i)));
		}
	}
}

void printWIT(uint8_t field)
{
	lcd_PrintNbr(fields[field].value);
	switch(fields[field].wit){
		case 0x00:
			lcd_printstr("$C"); //$ är fulhackat till grader-tecken i lcd.c
			break;
		case 0x01:
			lcd_printstr("%");
			break;
		case 0x02:
			lcd_printstr("Bar");
			break;
		case 0x03:
			lcd_printstr("mm");
			break;
		case 0x04:
			lcd_printstr("m/s");
			break;	
	}
}

void putvalues(uint8_t mu, uint8_t wit, uint8_t value)
{
	for(uint8_t i=0; i<4; i++){
		if(fields[i].mu == mu){
			if(fields[i].wit == wit){
				fields[i].value = value;
				break;
			}
		}
	}
}

/*
	function: printroot
	pMenupos is for portability
*/
void printroot(menu_t* pMenupos)
{
	/*lcd_clearscrn();
	
		lcd_setxy(0,1);
		lcd_printhex(getMyAddr());
	
	lcd_setxy(0,3);
	lcd_printstr("Field 1: ");
	//lcd_printhex(field_wit[0]);
	printWIT(0);
	
	lcd_setxy(0,4);
	lcd_printstr("Field 2: ");
	printWIT(1);
	
	lcd_setxy(0,5);
	lcd_printstr("Field 3: ");
	printWIT(2);
	
	lcd_setxy(0,6);
	lcd_printstr("Field 4: ");
	printWIT(3);
	*/
	ritadata();
	ritaruta();
}

/*
	Function: ritaruta
	Prints the main-window grid and titles.
*/
void ritaruta()
{
	lcd_setxy(38,0);
	lcd_printstr("MotorOla");

	lcd_setxy(0,0);
	lcd_printstr("Temp");
	lcd_setxy(0,3);
	lcd_printstr("Humid");
	lcd_setxy(0,5);
	lcd_printstr("Atm.");
	lcd_setxy((5*stdfont_width+2),1);
	lcd_printstr("Tendency");
	lcd_setxy((5*stdfont_width+2),4);
	lcd_printstr("Wind");

	lcd_hline((5*stdfont_width),92,7);
	
	lcd_vline((5*stdfont_width),0,63);
	lcd_vline(92,0,63);
	lcd_hline(0,(5*stdfont_width),(8*3-2));

	lcd_hline(0,(5*stdfont_width),39);

	lcd_hline((5*stdfont_width),92,55);
}

void ritadata()
{
	//kolla du_def.h, vädersträcken definierade.
	//kolla lcd_def.h, molnen definierade.
	uint8_t tmp = 0x04;
	/*
	 * Print the weather tendency field
	 */
	/* Define: TENDENCY
	 *	0x00 - Sun
	 *	0x01 - Sun/Cloud
	 *	0x02 - Cloud
	 *	0x03 - Cloud/Rain
	 *	0x04 - Sun/Cloud/Rain
	 */
	switch(tmp){
		case 0x00:
			lcd_setxy(46,2);
			lcd_printbm(sol_bits, sol_width, sol_height);
			break;
		case 0x01:
			lcd_setxy(46,2);
			lcd_printbm(solmoln_bits, solmoln_width, solmoln_height);
			lcd_setxy(46,3);
			lcd_printbm(moln_under_bits,moln_under_width,moln_under_height);
			break;
		case 0x02:
			lcd_setxy(46,2);
			lcd_printbm(moln_over_bits, moln_over_width, moln_over_height);
			lcd_setxy(46,3);
			lcd_printbm(moln_under_bits,moln_under_width,moln_under_height);
			break;
		case 0x03:
			lcd_setxy(46,2);
			lcd_printbm(moln_over_bits, moln_over_width, moln_over_height);
			lcd_setxy(46,3);
			lcd_printbm(regnmoln_bits,regnmoln_width,regnmoln_height);
			break;
		case 0x04:
			lcd_setxy(46,2);
			lcd_printbm(solmoln_bits, solmoln_width, solmoln_height);
			lcd_setxy(46,3);
			lcd_printbm(regnmoln_bits,regnmoln_width,regnmoln_height);
			break;
	}
	
	/*
	 * Print the wind field
	 *
	lcd_setxy((6*stdfont_width+2),6);
	lcd_PrintNbr(latestdata[WIND_SPEED]);
	
	lcd_setxy(((7*stdfont_width+2)+2*6),6);
	lcd_printstr("m/s");
	
	lcd_setxy((13*stdfont_width),6);
	switch(latestdata[WIND_DIR]){
		case 0x00:
			lcd_printstr("N");
			break;
		case 0x01:
			lcd_printstr("NE");
			break;
		case 0x02:
			lcd_printstr("E");
			break;
		case 0x03:
			lcd_printstr("SE");
			break;
		case 0x04:
			lcd_printstr("S");
			break;
		case 0x05:
			lcd_printstr("SW");
			break;
		case 0x06:
			lcd_printstr("W");
			break;
		case 0x07:
			lcd_printstr("NW");
			break;
		default:
			lcd_printstr("?");
			break;
	}
	
	/*
	 * Print the Temp field
	 */
	 
	 /*
	 * Print the Humidity field
	 */
	 
	 /*
	 * Print the Atmospheric preassure field
	 */
	 
	 /*
	 * Print the wind field
	 */
	 
	 /*
	 * Print the rest of the fields....
	 */
}

