/******************************************************************************
*                            radio.c
*                            ------
*   Detta r radio-modulen. Det r ett hjlpobjekt till comunication handlern.
*******************************************************************************/

/**
* Includes
*/
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "defines.h"
#include "radio_def.h"
#include "radio_prot.h"

#include "lcd_prot.h"
#include "lcd_def.h"

#include "spi_prot.h"
#include "spi_def.h"

#include "commHandler_def.h"
#include "commHandler_prot.h"

/**
* Private function prototypes
*/
void rad_reset(void);

/**
* Variables
*/
static uint8_t IRQ_Source;
static uint8_t nRF_Buffer[32];		// Buffer to hold data from 'nRF_Read_Buf()' function
static char receiverEnabled =0;
static uint8_t sinceLastFlags;
/**
* Constants
*/
const uint8_t TX_ADDRESS[TX_ADR_LENGTH]  = {0x69,0x50,0xc1,0x7e,0x00};


/***********************************************************************************
* Public functions
************************************************************************************/
/*ISR(PCINT1_vect)
{*/
void rad_IRQ(void)
{
	uint8_t temp,rx_pw,tmpval, pipe, fifostat;
	cli();
	
	
	temp = rad_getstat();
	
	if(temp & RX_DR){
		while(!(rad_getfifostat() & 0x01)){
			sinceLastFlags |= RX_DR;
			PORT_CE &= ~_BV(CE);
			IRQ_Source = RX_DR;
			tmpval = RX_DR;
			rad_putreg(&tmpval, STATUS, 1);
		
			pipe = ((temp>>1)&0x07);
			rad_getreg(&rx_pw, RX_PW_P0 + pipe, 1);
			RX_Data(nRF_Buffer, rx_pw);
		
			comReceive();
		}
		PORT_CE |= _BV(CE);		
	}
	if(temp & MAX_RT){
		sinceLastFlags |= MAX_RT;
		PORT_CE &= ~_BV(CE);
		IRQ_Source = MAX_RT;
		tmpval = MAX_RT;
		rad_putreg(&tmpval, STATUS, 1);
		PORT_CE |= _BV(CE);
	}
	if(temp & TX_DS){
		sinceLastFlags |= TX_DS;
		PORT_CE &= ~_BV(CE);
		IRQ_Source = TX_DS;
		tmpval = TX_DS;
		rad_putreg(&tmpval, STATUS, 1);
		rad_getreg(&fifostat,FIFO_STATUS,1);
		if(fifostat & 0x10)
			RX_Mode();
		PORT_CE |= _BV(CE);
	}
	if(temp & TX_FULL){
		sinceLastFlags |= TX_FULL;
		PORT_CE &= ~_BV(CE);
		rad_clean();
		PORT_CE |= _BV(CE);
	}
	/*TODO: sei();*/
}

void rad_init(void)
{
	uint8_t tmp[5];
	uint8_t d;
	
	sinceLastFlags = 0;
	setMyAddr(0x20);
	
	DDR_CE &= ~_BV(CE);
	
	d=0x03;
	rad_putreg(&d, SETUP_AW, 1);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = 0x00;
	rad_putreg(tmp, TX_ADDR, TX_ADR_LENGTH);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = 0x00;
	rad_putreg(tmp, RX_ADDR_P0, TX_ADR_LENGTH);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = getMyAddr();
	rad_putreg(tmp, RX_ADDR_P1, TX_ADR_LENGTH);

	d = 0x03;
	rad_putreg(&d ,EN_RXADDR, 1);

	d=TX_PLOAD_WIDTH;
	rad_putreg(&d, RX_PW_P0, 1);

	d=TX_PLOAD_WIDTH;
	rad_putreg(&d, RX_PW_P1, 1);
		
	d = 0x03;
	rad_putreg(&d, W_REGISTER(EN_AA), 1);

	d = 0x0f;
	rad_putreg(&d, W_REGISTER(SETUP_RETR), 1);
	
	d = 0x29;
	rad_putreg(&d, W_REGISTER(RF_CH), 1);

	d = 0x07;
	rad_putreg(&d, W_REGISTER(RF_SETUP), 1);

	d = 0x7f;
	rad_putreg(&d, W_REGISTER(CONFIG),1);

	rad_clean();
	
	DDR_CE |= _BV(CE);
}

/**
* Send data on initialized nRF24l01
* One high pulse(>10s) on CE will now send this
* packet and expext an acknowledgment from the RX device.
*/
uint8_t TX_Data(uint8_t* pPayload, uint8_t bytes)
{
	if(!(rad_getfifostat() & 0x20)){
		uint8_t ceval = PIN_CE & _BV(CE);
		PORT_CE &= ~_BV(CE);
		uint8_t cmd = W_TX_PAYLOAD;
		spiStart();
		spiTransieve(&cmd,1);
		spiTransieve(pPayload, bytes);
		spiEnd();
		if(ceval)
			PORT_CE |= _BV(CE);
		else{
			PORT_CE |= _BV(CE);
			_delay_us(20);
			PORT_CE &= ~_BV(CE);		
		}
		return 1;
	}
	return 0;
}

/**
* Retrieve data from initialized nRF24l01
*/
void RX_Data(uint8_t* pPayload, uint8_t bytes)
{
	uint8_t ceval = PIN_CE & _BV(CE);
	PORT_CE &= ~_BV(CE);
	uint8_t cmd = R_RX_PAYLOAD;
	spiStart();
	spiTransieve(&cmd,1);
	spiTransieve(pPayload, bytes);
	spiEnd();
	if(ceval)
		PORT_CE |= _BV(CE);
}

/**
* Function to read the status register of the radio.
*/
uint8_t rad_getstat() 
{
	uint8_t status = 0xFF;
	spiStart();
	spiTransieve(&status,1);
	spiEnd();
	return status;

}
uint8_t rad_getfifostat() 
{
	uint8_t stat = 0xFF;
	spiStart();
	rad_getreg(&stat, FIFO_STATUS, 1);
	spiEnd();
	return stat;

}
uint8_t rad_getconf() 
{
	uint8_t conf = 0xFF;
	spiStart();
	rad_getreg(&conf, CONFIG, 1);
	spiEnd();
	return conf;

}
uint8_t rad_getobs() 
{
	uint8_t conf = 0xFF;
	spiStart();
	rad_getreg(&conf, OBSERVE_TX, 1);
	spiEnd();
	return conf;

}

/**
* Puts the radio in power down mode.
*/
void rad_stdbymode(uint8_t mode)
{
	uint8_t conf;
	switch(mode){
		case 0:
			/*Total power down, time to power up: 150us*/
			PORT_CE &= ~_BV(CE);
			rad_getreg(&conf, CONFIG, 1);
			conf = conf & 0xFD;
			rad_putreg(&conf, CONFIG, 1);
			break;
		case 1:
			/*Standby mode I, time to RX/TX: 130us*/
			PORT_CE &= ~_BV(CE);
			rad_getreg(&conf, CONFIG, 1);
			conf = conf | 0x02;
			rad_putreg(&conf, CONFIG, 1);
			break;
		case 2:
		default:
			/* Standby mode II - "Not in standby at all, just nothing to send.",
			   time to TX: 130us */
			rad_getreg(&conf, CONFIG, 1);
			conf = conf | 0x02;
			rad_putreg(&conf, CONFIG, 1);
			PORT_CE |= _BV(CE);
			break;
	}
}

void rad_clean(void)
{
	uint8_t fifostat;
//	rs232_printstr("\r\nCleaning!!!\r\n");
	rad_getreg(&fifostat, FIFO_STATUS, 1);
	if(!(fifostat & 0x10)){
		/*TX_fifo not empty*/
		rad_cmd(FLUSH_TX);
	}
	if(!(fifostat & 0x01)){
		/*RX_fifo not empty*/
		if(IRQ_Source == IDLE){
			rad_cmd(FLUSH_RX);
		}
	}
}

void rad_setTXAddr(uint8_t toAddr)
{
	uint8_t tmp[5];
	uint8_t ceval = PIN_CE & _BV(CE);
	PORT_CE &= ~_BV(CE);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = toAddr;
	rad_putreg(tmp, TX_ADDR, TX_ADR_LENGTH);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = toAddr;
	rad_putreg(tmp, RX_ADDR_P0, TX_ADR_LENGTH);
	
	if(ceval)
		PORT_CE |= _BV(CE);
}

void rad_setRXAddr(uint8_t addr)
{
	uint8_t tmp[5];
	uint8_t ceval = PIN_CE & _BV(CE);
	PORT_CE &= ~_BV(CE);
	
	for(int i=0; i<4; i++)
		tmp[i] = TX_ADDRESS[i];
	tmp[4] = addr;
	rad_putreg(tmp, RX_ADDR_P1, TX_ADR_LENGTH);
		
	if(ceval)
		PORT_CE |= _BV(CE);
}

/**
* This function initializes one nRF24L01 device to
* TX mode, set TX address, select RF channel, datarate & TX pwr.
* PWR_UP is set, CRC(2 bytes) is enabled, & PRIM:TX.
*/
void TX_Mode(void)
{
	uint8_t data;
	uint8_t ceval = PIN_CE & _BV(CE);
	PORT_CE &= ~_BV(CE);
	
//	rs232_printstr("PTX\r\n");
	
	rad_getreg(&data, CONFIG, 1);
	data &= ~0x01;
	rad_putreg(&data, CONFIG, 1);

	if(ceval)
		PORT_CE |= _BV(CE);
	_delay_ms(10);
}


/**
* This function initializes one nRF24L01 device to
* RX Mode, set RX address, writes RX payload width,
* select RF channel, datarate & LNA HCURR.
* After init, CE is toggled high, which means that
* this device is now ready to receive a datapacket.
*/
void RX_Mode(void)
{
	uint8_t ceval = PIN_CE & _BV(CE);
	PORT_CE &= ~_BV(CE);
	
	uint8_t d;
	
//	rs232_printstr("PRX\r\n");
	
	rad_getreg(&d, CONFIG, 1);
	d |= 0x01;
	rad_putreg(&d, CONFIG, 1);

	if(ceval)
		PORT_CE |= _BV(CE);
	_delay_ms(10);
}

/***********************************************************************************
* Private functions
************************************************************************************/
void rad_reset()
{
	uint8_t d;
	
	PORT_CE &= ~_BV(CE);
	
	d = FLUSH_TX;
	spiStart();
	spiTransieve(&d,1);
	spiEnd();
	
	d = FLUSH_RX;
	spiStart();
	spiTransieve(&d,1);
	spiEnd();
	
	d = 0x08;
	rad_putreg(&d, W_REGISTER(CONFIG), 1);
	
	d = 0x3f;
	rad_putreg(&d, W_REGISTER(EN_AA), 1);
	
	d = 0x03;
	rad_putreg(&d, W_REGISTER(EN_RXADDR), 1);
	
	d = 0x03;
	rad_putreg(&d, W_REGISTER(SETUP_AW), 1);
	
	d = 0x03;
	rad_putreg(&d, W_REGISTER(SETUP_RETR), 1);
	
	d = 0x02;
	rad_putreg(&d, W_REGISTER(RF_CH), 1);
	
	d = 0x0f;
	rad_putreg(&d, W_REGISTER(RF_SETUP), 1);
}

/**
* Function to put one or more byte(s) in a register of the radio.
*/
uint8_t rad_putreg(uint8_t* pData, uint8_t reg, uint8_t bytes)
{	
	uint8_t status;
	uint8_t regis = W_REGISTER(reg);

	spiStart();
	status = spiTransieve(&regis, 1);// Select register to write to and read status uint8_t
	spiTransieve(pData, bytes);
	spiEnd();

	return(status);			// return nRF24L01 status byte
}

/**
* Function to read one or more byte(s) from a radio register.
*/
uint8_t rad_getreg(uint8_t* pBuff, uint8_t reg, uint8_t bytes)
{
	uint8_t status=0;
	uint8_t regis = R_REGISTER(reg);
	
	spiStart();
	status = spiTransieve(&regis, 1);			// Select register to write to and read status byte
	spiTransieve(pBuff, bytes);
	spiEnd();

	return(status);						// return nRF24L01 status byte
}

uint8_t rad_cmd(uint8_t to_flush)
{
	uint8_t status;
	
	spiStart();
	status = spiTransieve(&to_flush, 1);			// Select register to write to and read status byte
	spiEnd();
	return status;
}

uint8_t* rad_getbuff(void)
{
	return nRF_Buffer;
}

void setReceiverEnabled(uint8_t inVal)
{
	receiverEnabled = inVal;
}
uint8_t getReceiverEnabled(void)
{
	return receiverEnabled;
}


/***********************************************************************************
* Testcases
************************************************************************************/
void rad_test(void)
{

}

uint8_t getSinceLastFlags()
{
	return sinceLastFlags;
}
void setSinceLastFlags(uint8_t val)
{
	sinceLastFlags = val;
}
